﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace gdtLibrary
{
    public abstract class RenderableObject
    {
        // 그래픽 디바이스
        GraphicsDevice graphicsDevice;

        // 뷰포트가 null 값인지 체크해야 하는가
        protected bool viewportNullCheck { get; set; }

        // 뷰포트를 공급하기 위한 공급책
        protected Viewport viewport { get; set; }

        // 그래픽 장치
        public GraphicsDevice GraphicsDevice 
        { get { return GameMgr.Instance.GraphicsDevice; } }

        // 렌더링을 위해 이 함수가 호출된다.
        public bool Render(Viewport viewport, GameTime gameTime)
        {
            this.viewport = viewport;
            if (viewportNullCheck && viewport == null)
                return false;
            
            _beginRenderInternal(gameTime);
            _renderInternal(gameTime);
            _endRenderInternal(gameTime);

            return true;
        }

        // 실질적인 렌더링 작업을 한다.
        protected abstract void _renderInternal(GameTime gameTime);

        // 렌더링되기 전에 호출된다.
        protected abstract void _beginRenderInternal(GameTime gameTime);

        // 렌더링 된 후에 호출된다.
        protected abstract void _endRenderInternal(GameTime gameTime);
    }
}
