﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace gdtLibrary
{
    public class Viewport : RenderableObject
    {
        // 시점의 위치
        private Vector2 position;

        // 이 시점이 보고 있는 곳이 그려질 백 버퍼(렌더 타깃)
        private RenderTarget2D renderTarget;

        // 이 시점의 이름
        public string Name { get; set; }

        // 시점의 위치를 수정하기 위한 속성 필드
        public Vector2 Position
        {
            get { return position; }
            set
            {
                position = value;
                _updatePosition();
            }
        }

        // 렌더 타깃을 노출하기 위한 속성 필드
        public RenderTarget2D RenderTarget { get { return renderTarget; } }

        // 오퍼레이터
        public Viewport(string Name, Vector2 Position)
        {
            this.Name = Name;
            this.position = Position;
            this._updatePosition();

            this.viewport = this;
            this.viewportNullCheck = false;
        }

        // 시점의 위치가 바뀌면 호출되는 함수
        private void _updatePosition()
        {
            // _ 아직은 아무일 도 하지 않는다.
        }

        // 내부적으로 렌더링을 시작할때 초기화를 담당한다.
        protected override void _beginRenderInternal(GameTime gameTime)
        {
            // 렌더 타깃이 초기화 되지 않았다.
            if (renderTarget == null) // 게임 창 크기와 동일한 렌더 타깃 생성
                renderTarget = new RenderTarget2D(this.GraphicsDevice,
                    this.GraphicsDevice.Viewport.Width, this.GraphicsDevice.Viewport.Height);

            GraphicsDevice.SetRenderTarget(renderTarget); // 렌더 타깃을 장치에 설정.
        }

        // 내부적으로 렌더링을 시작할때 렌더링 종료를 담당한다.
        protected override void _endRenderInternal(GameTime gameTime)
        {
            // 렌더 타깃을 장치에서 해지함.
            GraphicsDevice.SetRenderTarget(null);
        }

        // 실질적인 렌더링 함수
        protected override void _renderInternal(GameTime gameTime)
        {
            // 개체 참조를 받아옴. 인스턴싱(복제)해 오는 것이 아니므로 메모리를 잡아머지는 않음.
            List<RenderableObject> _objects = GameMgr.Instance.Objects;

            // 뷰포트를 제외하고 모두 렌더링 시킴.
            foreach (RenderableObject _object in _objects)
            {
                if (!(_object is Viewport))
                    _object.Render(this, gameTime);
            }
        }
    }
}
